package com.xg.idcard;

import static com.cn.qsign.jsgzlibrary.utils.QsignNotaryApi.QSIGNREGISTERCODE.NORMALREGISTER;

import android.Manifest;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Base64;
import android.util.Log;

import com.cn.qsign.jsgzlibrary.utils.QsignNotaryApi;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaInterface;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CordovaWebView;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.FileOutputStream;
import java.util.HashSet;
import java.util.Set;

public class IdCardIdentifyPlugin extends CordovaPlugin {

    private static final String ACTION_INIT = "init";
    private static final String ACTION_REGISTER_NOTARY = "registerNotary";
    private static final String ACTION_HAVE_USER = "haveUser";
    private static final String ACTION_ACTIVE_USER_NOTARY = "activeUserNotary";
    private static final String ACTION_SET_WRITE_IMG_NOTARY = "setWriteImgNotary";
    private static final String ACTION_SET_FINGER_NOTARY = "setFingerNotary";
    private static final String ACTION_FORGET_PIN_NOTARY = "forgetPINNotary";
    private static final String ACTION_WEB_SIGN_NOTARY = "webSignNotary";
    private static final String ACTION_GET_FINGER_OPENED = "getFingerOpened";
    private static final String ACTION_QSIGN_LIVENESS_NOTARY = "qsignLivenessNotary";
    private static final String ACTION_IDENTIFY_NOTARY = "identifyNotary";
    private static final String ACTION_CLEAR_USER = "clearUser";
    private static final String ERROR_CODE_PARAM = "ERROR_CODE_PARAM";
    private static final String ERROR_CODE_INIT_SDK = "INIT_SDK_ERROR";
    private static final String ERROR_CODE_PERMISSION_ERROR = "ERROR_PERMISSION_MISSING";
    private static final String CODE_SUCCESS = "SUCCESS";
    private static final int REQUEST_PERMISSION_CODE = 21;
    private static final String[] PERMISSIONS = new String[]{
            Manifest.permission.CAMERA,
            Manifest.permission.READ_EXTERNAL_STORAGE,
            Manifest.permission.WRITE_EXTERNAL_STORAGE,
            Manifest.permission.RECORD_AUDIO
    };

    private CallbackContext globalCallbackContext = null;
    private JSONArray globalArgs = null;
    private String license = "";
    private String tag = "xugong";
    private boolean isActive = true;
    private boolean isProdEnv = false;

    @Override
    public void initialize(CordovaInterface cordova, CordovaWebView webView) {
        super.initialize(cordova, webView);
        String packageName = cordova.getActivity().getPackageName();
        if ("com.xcmg.app.dev".equals(packageName)) {
            license = "com.xcmg.app.dev-license.qsign-android";
            isProdEnv = false;
        } else {
            license = "com.xcmg.app-license.qsign-android";
            isProdEnv = true;
        }
        boolean result = QsignNotaryApi.getInstance().init(cordova.getActivity().getApplication(), license, tag, isActive, isProdEnv);
        Log.d("IdCardIdentifyPlugin", "result = " + result);
    }

    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
        if (ACTION_INIT.equals(action)) {
            initSDK(args, callbackContext);
            return true;
        } else if (ACTION_REGISTER_NOTARY.equals(action)) {
            registerNotary(args, callbackContext);
            return true;
        } else if (ACTION_HAVE_USER.equals(action)) {
            haveUser(args, callbackContext);
            return true;
        } else if (ACTION_ACTIVE_USER_NOTARY.equals(action)) {
            activeUserNotary(args, callbackContext);
            return true;
        } else if (ACTION_SET_WRITE_IMG_NOTARY.equals(action)) {
            setWriteImgNotary(args, callbackContext);
            return true;
        } else if (ACTION_SET_FINGER_NOTARY.equals(action)) {
            setFingerNotary(callbackContext);
            return true;
        } else if (ACTION_FORGET_PIN_NOTARY.equals(action)) {
            forgetPINNotary(callbackContext);
            return true;
        } else if (ACTION_WEB_SIGN_NOTARY.equals(action)) {
            webSignNotary(args, callbackContext);
            return true;
        } else if (ACTION_CLEAR_USER.equals(action)) {
            clearUser(callbackContext);
            return true;
        } else if (ACTION_GET_FINGER_OPENED.equals(action)) {
            getFingerOpened(callbackContext);
            return true;
        } else if (ACTION_QSIGN_LIVENESS_NOTARY.equals(action)) {
            qsignLivenessNotary(args, callbackContext);
            return true;
        } else if (ACTION_IDENTIFY_NOTARY.equals(action)) {
            identifyNotary(args, callbackContext);
            return true;
        }
        return false;
    }

    private void qsignLivenessNotary(JSONArray args, final CallbackContext callbackContext) {
        String name = "";
        String IDNo = "";
        try {
            JSONObject params = args.getJSONObject(0);
            name = params.getString("name");
            IDNo = params.getString("IDNo");
        } catch (JSONException e) {
            callbackContext.error(getResult("参数错误：" + e, ERROR_CODE_PARAM));
            return;
        }
        QsignNotaryApi.getInstance().qsignLivenessNotary(cordova.getActivity(), name, IDNo, new QsignNotaryApi.notaryCallbackListener() {
            @Override
            public void onSuccess() {
                callbackContext.success(getResult("操作成功", CODE_SUCCESS));
            }

            @Override
            public void onError(QsignNotaryApi.QSIGNERRORCODE code, String msg) {
                callbackContext.error(getResult(msg, code.name(), null));
            }
        });
    }

    private void initSDK(JSONArray args, final CallbackContext callbackContext) {
        String license = "";
        String tag = "";
        boolean isActive = true;
        boolean isProdEnv = false;
        try {
            JSONObject params = args.getJSONObject(0);
            license = params.getString("license");
            tag = params.getString("tag");
            isActive = params.getBoolean("isActive");
            isProdEnv = params.getBoolean("isProdEnv");
        } catch (JSONException e) {
            callbackContext.error(getResult("参数错误：" + e, ERROR_CODE_PARAM));
            return;
        }

        /*
          参数2 license
          参数3 tag
          参数4 是否激活  true
          参数5 服务器环境  true：正式服务器环境 false:测试服务器环境hnsdk
        */
        boolean result = QsignNotaryApi.getInstance().init(cordova.getActivity().getApplication(),
                license, tag,
                isActive, isProdEnv);
        if (result) {
            callbackContext.success(getResult("初始化成功", CODE_SUCCESS));
        } else {
            callbackContext.error(getResult("初始化失败，请检查参数等数据是否异常", ERROR_CODE_INIT_SDK));
        }
    }

    /**
     * 实名注册并设置PIN接口
     */
    private void registerNotary(JSONArray args, final CallbackContext callbackContext) {
        String bizNo = "";
        String name = "";
        String IDNo = "";
        boolean disallowModify = true;
        try {
            JSONObject params = args.getJSONObject(0);
            bizNo = params.getString("bizNo");
            name = params.getString("name");
            IDNo = params.getString("IDNo");
            disallowModify = params.getBoolean("disallowModify");
        } catch (JSONException e) {
            callbackContext.error(getResult("参数错误：" + e, ERROR_CODE_PARAM));
            return;
        }
        Set<String> requiredPermissions = getRequiredPermissions();
        if (requiredPermissions.size() > 0) {
            String[] permissions = requiredPermissions.toArray(new String[]{});
            globalCallbackContext = callbackContext;
            globalArgs = args;
            cordova.requestPermissions(this, REQUEST_PERMISSION_CODE, permissions);
            return;
        }

        //实名注册 并生成协同密钥 并增加签署口令设置

        /*
          参数2：bizNO 唯一编码 可以设置为身份证号，或唯一随机数
          参数3：姓名
          参数4：身份证号
          参数5：是否不可修改  为true,即姓名和身份证不能修改并且不能为空
         */
        QsignNotaryApi.getInstance().registerNotary(cordova.getActivity(), bizNo, name, IDNo, disallowModify, new QsignNotaryApi.registerCallbackListener() {
            @Override
            public void onSuccess(QsignNotaryApi.QSIGNREGISTERCODE code, String idnumber, String name, String phone) {
                JSONObject data = new JSONObject();
                try {
                    data.put("IDNo", idnumber);
                    data.put("name", name);
                    data.put("phone", phone);
                } catch (JSONException ignored) {
                }
                String message = code == NORMALREGISTER ? "成功" : "信息提交成功，等待人工审核";
                callbackContext.success(getResult(message, code.name(), data));
            }

            @Override
            public void onError(QsignNotaryApi.QSIGNERRORCODE code, String msg) {
                callbackContext.error(getResult(msg, code.name(), null));
            }
        });
    }

    private void identifyNotary(JSONArray args, final CallbackContext callbackContext) {
        String name = "";
        String IDNo = "";
        String phone = "";

        try {
            JSONObject params = args.getJSONObject(0);
            name = params.getString("name");
            IDNo = params.getString("IDNo");
            phone = params.getString("phone");
        } catch (JSONException e) {
            callbackContext.error(getResult("参数错误：" + e, ERROR_CODE_PARAM));
            return;
        }
        Set<String> requiredPermissions = getRequiredPermissions();
        if (requiredPermissions.size() > 0) {
            String[] permissions = requiredPermissions.toArray(new String[]{});
            globalCallbackContext = callbackContext;
            globalArgs = args;
            cordova.requestPermissions(this, REQUEST_PERMISSION_CODE, permissions);
            return;
        }

        //实名认证 并生成协同密钥

        /*
          参数2：bizNO 唯一编码 可以设置为身份证号，或唯一随机数
          参数3：姓名
          参数4：身份证号
          参数5：是否不可修改  为true,即姓名和身份证不能修改并且不能为空
         */
        QsignNotaryApi.getInstance().identifyNotary(cordova.getActivity(), name, IDNo, phone, new QsignNotaryApi.registerCallbackListener() {
            @Override
            public void onSuccess(QsignNotaryApi.QSIGNREGISTERCODE code, String idnumber, String name, String phone) {
                JSONObject data = new JSONObject();
                try {
                    data.put("IDNo", idnumber);
                    data.put("name", name);
                    data.put("phone", phone);
                } catch (JSONException ignored) {
                }
                String message = code == NORMALREGISTER ? "成功" : "信息提交成功，等待人工审核";
                callbackContext.success(getResult(message, code.name(), data));
            }

            @Override
            public void onError(QsignNotaryApi.QSIGNERRORCODE code, String msg) {
                callbackContext.error(getResult(msg, code.name(), null));
            }
        });
    }


    private void activeUserNotary(JSONArray args, final CallbackContext callbackContext) {
        String bizNo = "";
        try {
            JSONObject params = args.getJSONObject(0);
            bizNo = params.getString("bizNo");
        } catch (JSONException e) {
            callbackContext.error(getResult("参数错误：" + e, ERROR_CODE_PARAM));
            return;
        }
        QsignNotaryApi.getInstance().activeUserNotary(cordova.getActivity(), bizNo, new QsignNotaryApi.activeCallbackListener() {
            @Override
            public void onSuccess(String idnumber, String name, String phone) {
                JSONObject result = new JSONObject();
                try {
                    result.put("IDNo", idnumber);
                    result.put("name", name);
                    result.put("phone", phone);
                } catch (JSONException ignored) {
                }
                callbackContext.success(getResult("激活成功", CODE_SUCCESS, result));
            }

            @Override
            public void onError(QsignNotaryApi.QSIGNERRORCODE code, String msg) {
                callbackContext.error(getResult(msg, code.name()));
            }
        });
    }

    private void haveUser(JSONArray args, CallbackContext callbackContext) {
        String bizNo = "";
        try {
            JSONObject params = args.getJSONObject(0);
            bizNo = params.getString("bizNo");
        } catch (JSONException e) {
            callbackContext.error(getResult("参数错误：" + e, ERROR_CODE_PARAM));
            return;
        }
        Boolean haveUser = QsignNotaryApi.getInstance().haveUser(cordova.getActivity(), bizNo);
        JSONObject result = new JSONObject();
        try {
            result.put("haveUser", haveUser);
        } catch (JSONException ignored) {
        }
        callbackContext.success(getResult("成功", CODE_SUCCESS, result));
    }


    private void setWriteImgNotary(JSONArray args, final CallbackContext callbackContext) {
        String bizNo = "";
        try {
            JSONObject params = args.getJSONObject(0);
            bizNo = params.getString("bizNo");
        } catch (JSONException e) {
            callbackContext.error(getResult("参数错误：" + e, ERROR_CODE_PARAM));
            return;
        }
        QsignNotaryApi.getInstance().setWriteImgNotary(cordova.getActivity(), bizNo, new QsignNotaryApi.notaryCallbackListener() {
            @Override
            public void onSuccess() {
                callbackContext.success(getResult("成功", CODE_SUCCESS));
            }

            @Override
            public void onError(QsignNotaryApi.QSIGNERRORCODE code, String msg) {
                callbackContext.error(getResult(msg, code.name()));
            }
        });
    }

    private void setFingerNotary(final CallbackContext callbackContext) {
        QsignNotaryApi.getInstance().setFingerNotary(cordova.getActivity(), new QsignNotaryApi.notaryCallbackListener() {
            @Override
            public void onSuccess() {
                JSONObject result = new JSONObject();
                try {
                    result.put("isFingerOpened", QsignNotaryApi.getInstance().getFingerOpened(cordova.getActivity()));
                } catch (JSONException ignored) {
                }
                callbackContext.success(getResult("成功", CODE_SUCCESS, result));
            }

            @Override
            public void onError(QsignNotaryApi.QSIGNERRORCODE code, String msg) {
                callbackContext.error(getResult(msg, code.name()));
            }
        });
    }

    private void forgetPINNotary(final CallbackContext callbackContext) {
        QsignNotaryApi.getInstance().forgetPINNotary(cordova.getActivity(), new QsignNotaryApi.notaryCallbackListener() {
            @Override
            public void onSuccess() {
                callbackContext.success(getResult("成功", CODE_SUCCESS));
            }

            @Override
            public void onError(QsignNotaryApi.QSIGNERRORCODE code, String msg) {
                callbackContext.error(getResult(msg, code.name()));
            }
        });
    }

    private void webSignNotary(JSONArray args, final CallbackContext callbackContext) {
        String url = "";
        try {
            JSONObject params = args.getJSONObject(0);
            url = params.getString("url");
        } catch (JSONException e) {
            callbackContext.error(getResult("参数错误：" + e, ERROR_CODE_PARAM));
            return;
        }
        QsignNotaryApi.getInstance().webSignNotary(cordova.getActivity(), url, new QsignNotaryApi.notaryCallbackListener() {
            @Override
            public void onSuccess() {
                callbackContext.success(getResult("成功", CODE_SUCCESS));
            }

            @Override
            public void onError(QsignNotaryApi.QSIGNERRORCODE code, String msg) {
                callbackContext.error(getResult(msg, code.name()));
            }
        });
    }

    private void clearUser(CallbackContext callbackContext) {
        QsignNotaryApi.getInstance().ClearUser(cordova.getActivity());
        callbackContext.success(getResult("清除成功", CODE_SUCCESS));
    }


    private void getFingerOpened(CallbackContext callbackContext) {
        JSONObject result = new JSONObject();
        try {
            result.put("isFingerOpened", QsignNotaryApi.getInstance().getFingerOpened(cordova.getActivity()));
        } catch (JSONException ignored) {
        }
        callbackContext.success(getResult("成功", CODE_SUCCESS, result));

    }

    private JSONObject getResult(String message, String code) {
        return getResult(message, code, null);
    }

    private JSONObject getResult(String message, String code, JSONObject data) {
        JSONObject out = new JSONObject();
        try {
            if (code != null) {
                out.put("code", code);
            }
            out.put("message", message);
            if (data != null) {
                out.put("data", data);
            }
        } catch (JSONException ignored) {
        }
        return out;
    }

    @Override
    public void onRequestPermissionResult(int requestCode, String[] permissions, int[] grantResults) throws JSONException {
        super.onRequestPermissionResult(requestCode, permissions, grantResults);
        if (requestCode == REQUEST_PERMISSION_CODE) {
            Set<String> requiredPermissions = getRequiredPermissions();
            if (requiredPermissions.size() == 0) {
                registerNotary(globalArgs, globalCallbackContext);
            } else {
                StringBuilder sb = new StringBuilder();
                boolean hasStorageTint = false;
                JSONArray missPermissionsArray = new JSONArray();
                for (String s : requiredPermissions) {
                    if (Manifest.permission.WRITE_EXTERNAL_STORAGE.equals(s) || Manifest.permission.READ_EXTERNAL_STORAGE.equals(s)) {
                        if (!hasStorageTint) {
                            sb.append("读写存储权限，");
                            missPermissionsArray.put("读写存储权限");
                            hasStorageTint = true;
                        }
                    }
                    if (Manifest.permission.CAMERA.equals(s)) {
                        sb.append("相机权限，");
                        missPermissionsArray.put("相机权限");
                    }
                    if (Manifest.permission.RECORD_AUDIO.equals(s)) {
                        missPermissionsArray.put("录音权限");
                        sb.append("录音权限，");
                    }
                }
                JSONObject result = new JSONObject();
                result.put("missPermissions", missPermissionsArray);
                globalCallbackContext.error(getResult("缺少权限：" + sb, ERROR_CODE_PERMISSION_ERROR, result));
            }
        }
    }

    private Set<String> getRequiredPermissions() {
        Set<String> sets = new HashSet<String>();
        for (String permission : IdCardIdentifyPlugin.PERMISSIONS) {
            boolean b = cordova.hasPermission(permission);
            if (!b) {
                sets.add(permission);
            }
        }
        return sets;
    }

    private String saveImage(String base64String) {
        byte[] decode = Base64.decode(base64String, Base64.DEFAULT);
        Bitmap bitmap = BitmapFactory.decodeByteArray(decode, 0, decode.length);
        File appDir = new File(cordova.getActivity().getApplicationContext().getFilesDir(), "QSign_Face");
        if (!appDir.exists()) {
            appDir.mkdir();
        }
        String fileName = System.currentTimeMillis() + ".jpg";
        File file = new File(appDir, fileName);
        try {
            FileOutputStream fos = new FileOutputStream(file);
            bitmap.compress(Bitmap.CompressFormat.JPEG, 100, fos);
            fos.flush();
            fos.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return file.getPath();
    }
}
